--[[
*****************************************************
Script for USB IR-Remote "CanoRemote" in "Fast Mode"
Version: 1.5
URL: http://www.canoremote.de
Uses the "CHDK fast mode" of the USB-IR-receiver!!!
Requires CHDK 1.3 >= v3442 and a processor >= Digic-3
Date: 2015/03/23
Licence: GPL
Authors: f_m_b and Werner_O
Thx msl
*****************************************************
@title CanoRemote Fast 1.5
@chdk_version 1.3

@param     a Enable Auto-Mode USB-Port
  @default a 1
  @range   a 0 1

@param     m AFL-Mode for Key Down
  @default m 0
  @range   m 0 1

@param     f Wait for Flash
  @default f 0
  @range   f 0 1

@param     z Zoom Step Time (10ms)
  @default z 3
  @range   z 1 20

@param     t Timeout Cont. Zoom (100ms)
  @default t 6
  @range   t 3 10
]]

--Define values for various variables:
usbautomode = a                  --Definition automode for USB port
aflmode = m                      --Definition alternate mode for key DOWN (AFL instead Halfshoot)
aflstate = 0                     --First definition for AFL state
flashmode = f                    --Definition flashmode (wait for flash before exposure)
flashmode_real = 0               --First Definition flashmode_real; might be changed later
delay_left = 30
delay_right = 60
videostate = get_video_button()  --Is a separate videobutton available (1 = Yes; 0 = No)?

--Define start values for zoom functions:
zoomsteptime = z * 10            --Definition zoomtime in zoom step mode
max_zoom = get_zoom_steps() - 1  --Definition max. zoom position (Tele)
zoomposition = get_zoom()        --Current zoom position
zoominpercent = 1                --Predefinition zoom position in percent; will be actualized later
KEY_TIMEOUT = t * 100            --Definition timeout for continuos zoom
countIn  = get_tick_count()      --Time reference for zoomIn(); will be actualized later
countOut = get_tick_count()      --Time reference for zoomOut(); will be actualized later

--Define possible values for variable "state" (actual script-state-mode):
STATE_IDLE = 0        --No key is pressed currently
STATE_IN_STEP = 1     --No key is pressed currently; last pressed key was zoom-in
STATE_IN_CONT = 2     --Key zoom-in is pressed currently
STATE_OUT_STEP = 3    --No key is pressed currently; last pressed key was zoom-out
STATE_OUT_CONT = 4    --Key zoom-out is pressed currently
STATE_SHOOT_HALF = 5  --Key halfshoot is pressed currently with locked AF- and AE-settings

--Define variable "state" at start-up:
state = STATE_IDLE

--Define console layout:
set_console_layout(0, 0, 31, 10)

--Define subfunktion restore() used at script abort:
function restore()
    cls()
    print("Script aborted:")
    if aflstate == 1 then
        set_aflock(0)
        print("AFL disabled")
    elseif state == STATE_SHOOT_HALF then
        release("shoot_half")
    end
    --Disable USB high speed mode
    if type(set_remote_timing) == "function" then set_remote_timing(0)
    elseif type(enable_remote_hp_timer) == "function" then enable_remote_hp_timer(0)
    elseif type(enable_highspeed_usb) == "function" then enable_highspeed_usb(0)
    end
    print("USB High speed mode disabled")
    if usbautomode == 1 then
        set_config_value(121,0)      --Disable USB remote function
        print("USB remote function disabled")
    end
    set_console_layout(0, 0, 25, 5)  --Restore default console layout
end

--Definition subfunction state_print() to display current script mode:
function state_print()
    cls()
    if scriptstate == 1 then         --True after pressing key MENU; shows additional script states
        print("CURRENT SCRIPT STATE:")
        if usbautomode == 1 then
            print("USB remote function was enabled")
        else
            print("USB auto-mode is disabled")
        end
        if aflmode == 0 then
            print("Halfshoot mode enabled:")
            if state == STATE_SHOOT_HALF then
                print("Halfshoot is locked")
            else
                print("Halfshoot is unlocked")
            end
        else
            print("AFL-mode enabled:")
            if aflstate == 1 then
                print("Focus is locked")
            else
                print("Focus is unlocked")
            end
        end
        if flashmode == 1 then
            if get_flash_mode() ~= 2 then
                print("Flash mode enabled:")
                print("Flash is available")
            else
                print("Flash mode enabled:")
                print("Flash is switched off")
            end
        end
        if videostate == 1 then
            print("Video recording is possible")
        else
            print("Video recording isn't possible")
        end
    end  
    zoominpercent = (get_zoom() * 100 / max_zoom)
    print(string.format("Zoom position = %d Percent", zoominpercent))
    if state == STATE_IDLE then
        if aflstate == 1 then
            print("Mode = Idle; Focus is locked")
        else
            print("Mode = Idle")
        end
    elseif state == STATE_OUT_STEP then
        print(string.format("Mode = Zoom out step for %d ms", KEY_TIMEOUT))
    elseif state == STATE_IN_STEP then
        print(string.format("Mode = Zoom in step for %d ms", KEY_TIMEOUT))
    elseif state == STATE_OUT_CONT then
        print("Mode = Zoom out continuos")
    elseif state == STATE_IN_CONT then
        print("Mode = Zoom in continuos")
    elseif state == STATE_SHOOT_HALF then
        print("Mode = Halfshoot fixed")
    end
    print("Waiting for USB impulse")
    print("(Press MENU at CAM to EXIT)")
    if scriptstate == 1 then
        sleep(3000)
        console_redraw()
        sleep(3000)
        console_redraw()
        sleep(3000)
    end
    scriptstate = 0
end

--Definition subfunction flashfunction() - check if flash is available:
function flashfunction()
    flashmode_current = get_flash_mode()
    if flashmode_current ~= flashmode_last then
        cls()
        if flashmode_current == 2 then          --True if flash is switched off
            flashmode_real = 0
            print("Flash is switched off:")
            print("Flashmode disabled")
        else
            flashmode_real = 1
            print("Flash is switched on:")
            print("Flashmode enabled")
        end
        sleep(3000)
        flashmode_last = flashmode_current
    end
end

--Definition subfunction for key LEFT:
function command_zoomOut()
    if state == STATE_OUT_CONT then
        release("zoom_out")
        state = STATE_IDLE
        state_print()
    else
        if state == STATE_SHOOT_HALF then
            release("shoot_half")
            sleep(10)
            state = STATE_IDLE
        elseif state == STATE_IN_CONT then
            release("zoom_in")
            sleep(10)
            state = STATE_IDLE
        end
        if get_zoom() > 0 then               --True if zoom position is higher than min. zoom
            if state == STATE_OUT_STEP then  --Activate continous zoom-out
                press("zoom_out")
                state = STATE_OUT_CONT
            else                             --Activate a single zoom-out step
                countOut = get_tick_count()
                press("zoom_out")
                sleep(zoomsteptime)
                release("zoom_out")
                state = STATE_OUT_STEP
            end
            state_print()
        else
            state = STATE_IDLE
            cls()
            print("Zoom out not possible")
            print("Mode = Idle")
            print("(Press MENU at CAM to EXIT)")
            sleep(3000)
            cls()
        end
    end
end

--Definition subfunction for key RIGHT:
function command_zoomIn()
    if state == STATE_IN_CONT then
        release("zoom_in")
        state = STATE_IDLE
        state_print()
    else
        if state == STATE_SHOOT_HALF then
            release("shoot_half")
            sleep(10)
            state = STATE_IDLE
        elseif state == STATE_OUT_CONT then
            release("zoom_out")
            sleep(10)
            state = STATE_IDLE
        end
        if get_zoom() < max_zoom then       --True if zoom position is lower than mx. zoom
            if state == STATE_IN_STEP then  --Activate continous zoom-in
                press("zoom_in")
                state = STATE_IN_CONT
            else                            --Activate a single zoom-in step
                countIn = get_tick_count()
                press("zoom_in")
                sleep(zoomsteptime)
                release("zoom_in")
                state = STATE_IN_STEP
            end
            state_print()
        else
            state = STATE_IDLE
            cls()
            print("Zoom in not possible")
            print("Mode = Idle")
            print("(Press MENU at CAM to EXIT)")
            sleep(3000)
            cls()
        end
    end
end

--Definition subfunction for key UP:
function startStopVideo()
    if videostate == 1 then                   --Is a separate video button available?
        if state == STATE_OUT_CONT then       --Releases a pressed key zoom-out
            release("zoom_out") 
            sleep(10)
        elseif state == STATE_IN_CONT then    --Releases a pressed key zoom-in
            release("zoom_in")
            sleep(10)
        elseif state == STATE_SHOOT_HALF then --Releases a pressed shoothalf
            release("shoot_half")
            sleep(10)
        end
        if get_movie_status() <= 1 then       --True at stopped/paused video record
            click("video")                    --Start video record
            sleep(100)
        else
            click("video")                    --Stop video record
            repeat sleep(100) until get_movie_status() <= 1   --Wait until videoprocessing has finished
        end
        state = STATE_IDLE
        state_print()
    else                                     --Print error message
        print("No video button found:")
        print("Video record is not available")
        sleep(3000)
        state = STATE_IDLE
        state_print()
    end
end

--Definition subfunction for key DOWN in standard script mode (Halfshoot instead AFL):
function command_shootHalf()
    if state == STATE_OUT_CONT then
        release("zoom_out")
        sleep(10)
    elseif state == STATE_IN_CONT then
        release("zoom_in")
        sleep(10)
    end
    if state == STATE_SHOOT_HALF then
        release "shoot_half"
        state = STATE_IDLE
    else
        if get_movie_status() <= 1 then    --True at stopped/paused video record
            press("shoot_half")
            repeat sleep(10) until get_shooting() == true
            state = STATE_SHOOT_HALF
        else                               --True while active video recording/processing
            if aflstate == 0 then          --True if no AFL-lock currently active
                set_aflock(1)              --Enable AFL
                aflstate = 1
            else
                set_aflock(0)              --Disable AFL
                aflstate = 0
            end
            state = STATE_IDLE
        end
    end
    state_print()
end

--Definition subfunction for key DOWN in alternate script mode (AFL instead Halfshoot):
function command_afl()
    if state == STATE_OUT_CONT then        --Releases a pressed key zoom-out
        release("zoom_out")
        sleep(10)
    elseif state == STATE_IN_CONT then     --Releases a pressed key zoom-in
        release("zoom_in")
        sleep(10)
    end
    if aflstate == 0 then    --True if no AFL-lock currently active
        if get_movie_status() <= 1 then    --True at stopped/paused video record
            press("shoot_half")
            repeat sleep(10) until get_shooting() == true
            set_aflock(1)    --Enable AFL
            release("shoot_half")
            aflstate = 1
        else
            set_aflock(1)
            aflstate = 1
        end
    else
        set_aflock(0)        --Disable AFL
        aflstate = 0
    end
    state = STATE_IDLE
    state_print()
end

--Definition subfunction for key PLAY/PAUSE:
function command_shootFull()
    if state == STATE_OUT_CONT then     --Releases a pressed key zoom-out
        release("zoom_out")
        sleep(10)
        state = STATE_IDLE
    elseif state == STATE_IN_CONT then  --Releases a pressed key zoom-in
        release("zoom_in")
        sleep(10)
        state = STATE_IDLE
    end
    if get_movie_status() <= 1 then     --True if video record is stopped/paused
        if flashmode_real == 1 and get_movie_status() <= 1 then   --Wait for flash if enabled
            if get_flash_ready() == 0 then
                cls()
                print("Waiting for flash...")
                repeat sleep(10) until get_flash_ready() == 1
            end
        end
        if state == STATE_SHOOT_HALF then   --True if halfshoot is currently pressed
            count = get_exp_count()
            press("shoot_full_only")
            repeat sleep(10) until get_exp_count() ~= count  --Wait until new exposure has been saved succesfully
            release("shoot_full_only")
            release ("shoot_half")
        else                                --True if halfshoot is not active
            count = get_exp_count()
            shoot()
            repeat sleep(10) until get_exp_count() ~= count  --Wait until new exposure has been saved succesfully
        end
        state = STATE_IDLE
    else                                    --True while video recording/processing
        cls()
        print("Shooting not possible")
        print("while video recording!")
        sleep(3000)
        state = STATE_IDLE
    end
    state_print()
end

--Definition subfunction for key MENU:      --Show script state; Key MENU can also be used to wakeup the USB receiver!
function script_state()
    cls()
    scriptstate = 1
    state_print()
end

--SCRIPT START:

--Try to activate USB high speed mode - available since CHDK 1.3 >= v3442:
high_speed = false
if type(set_remote_timing) == "function" then high_speed = set_remote_timing(1000)
elseif type(enable_remote_hp_timer) == "function" then high_speed = enable_remote_hp_timer(1000)
elseif type(enable_highspeed_usb) == "function" then enable_highspeed_usb(1000) high_speed = true
end

if high_speed then
    print("USB high speed mode enabled")
    --Auto-activate USB remote function if wanted:
    if usbautomode == 1 then
        set_config_value(121,1)
    end    
    --Initialize variables for flash mode:
    if flashmode == 1 then
        flashmode_current = get_flash_mode()
        flashmode_last = flashmode_current
        if get_flash_mode() ~= 2 then   --True if flash isn't switched off
            flashmode_real = 1
        end
    end
    scriptstate = 1
    state_print()
    
    --MAIN LOOP:
    repeat
        if state == STATE_OUT_CONT and get_zoom() == 0 then
            release("zoom_out")     --Automatically release key zoom-out at reached min. zoom position
            state = STATE_IDLE
            state_print()
        elseif state == STATE_IN_CONT and get_zoom() == max_zoom then
            release("zoom_in")      --Automatically release key zoom-in at reached max. zoom position
            state = STATE_IDLE
            state_print()
        elseif state == STATE_OUT_STEP then
            if (get_tick_count() - countOut - 30) > KEY_TIMEOUT then    --The -30 compensates the 30ms USB-delay-time for key LEFT
                state = STATE_IDLE  --Disable state mode STATE_OUT_STEP after timeout for continous zoom
                state_print()
            end
        elseif state == STATE_IN_STEP then
            if (get_tick_count() - countIn - 60) > KEY_TIMEOUT then    --The -60 compensates the 60ms USB-delay-time for key RIGHT
                state = STATE_IDLE  --Disable state mode STATE_IN_STEP after timeout for continous zoom
                state_print()
            end
        end
        if flashmode == 1 and get_movie_status() <= 1 then
            flashfunction()
        end

        x = get_usb_power()        --Measurement of a recognized USB impulse
        if x > 0 then
            --Analyse the USB impulse lengths for further commands; starting from the lowest
            --defined impulse length step by step up to the highest defined impulse length:
            if x <= 42 then
                command_zoomOut()
            elseif x <=  72 then
                command_zoomIn()
            elseif x < 102 then
                startStopVideo()
            elseif x <= 132 then
                if aflmode == 0 and get_movie_status() <= 1 then   --True in standard script mode (Halfshoot instead AFL) with stopped/paused video recording
                    command_shootHalf()
                else               --True in alternate script mode (AFL instead Halfshoot)
                    command_afl()
                end
            elseif x <= 162 then
                command_shootFull()
            else                   --True if REMOTE-key(!) MENU has been pressed
                scriptstate = 1
                script_state()
            end
        end
    until is_pressed("menu")       --True if CAM-key(!) MENU has been pressed
else
    cls()
    print("ERROR:")
    print("This script requires CHDK 1.3")
    print("version 3442 or higher!")
    sleep(3000)
	console_redraw()
end

restore()
